# This constructs a minimal initramfs userspace containing `signify`,
# `lvm`, `dmsetup`, `cryptsetup`, and `kexec`, plus the
# modules from the `kernel` derivation, and creates an
# initramfs-formatted `cpio` archive from their contents.
{ lib
, nixpkgsOnBuildForHost
, nixpkgsOnBuildForBuild
, kernel
, bootScript ? ../boot.sh   # symlinked to /init if non-null

# a list of paths (relative to ${kernel}/lib/modules/*/kernel) to modules .ko
# files which should be included in the initrd
, modules ? [ ]

, withBusybox ? true
, withSignify ? true
# Inclusion of these is temporarily disabled.  I had a bunch of
# ugly space-saving hacks that were removed during the pre-release
# cleanup, and it turns out that without those hacks there isn't enough
# space for the large userspace tools.  Once I clean up and
# reinstate those hacks I will reenable this.
, withLvm ? false
, withCryptsetup ? false
, withKexec ? false
, withNvramTool ? false # nixpkgsOnBuildForHost.stdenv.hostPlatform.isx86
}:

let
  inherit (nixpkgsOnBuildForBuild) findutils cpio;
  inherit (nixpkgsOnBuildForHost.pkgsStatic) stdenv busybox signify lvm2 cryptsetup kexec-tools;
  nvramtool = nixpkgsOnBuildForHost.pkgsStatic.nvramtool.overrideAttrs(a: {
    NIX_CFLAGS_COMPILE = "-D__GLIBC__";
  });
in stdenv.mkDerivation {
  name = "initramfs.cpio";
  dontUnpack = true;
  dontFixup = true;

  modulesList = (lib.concatMapStringsSep "\n" (m: "${kernel.version}/kernel/" + m) modules)+"\n";
  passAsFile = [ "modulesList" ];

  buildPhase = ''
    mkdir build
    build=$(pwd)/build

    runHook preBuild

    mkdir -p $build/lib/modules
    pushd ${kernel}/lib/modules/
      cat $modulesListPath | ${cpio}/bin/cpio -p -d $build/lib/modules
      popd

    mkdir -p $build/usr
    ln -s bin $build/sbin
    ln -s ../bin $build/usr/bin
    ln -s ../sbin $build/usr/sbin
  '' + lib.optionalString withBusybox ''
    cp -r ${busybox}/bin              $build/bin
    chmod -R u+w $build/bin
  '' + lib.optionalString (bootScript != null) ''
    cp    ${bootScript} $build/boot.sh
    chmod +x $build/boot.sh
    ln -s boot.sh $build/init
    chmod +x $build/sbin/init
  '' + lib.optionalString withSignify ''
    cp    ${signify}/bin/signify      $build/bin/
  '' + lib.optionalString withLvm ''
    cp ${lib.getBin lvm2}/bin/lvm $build/bin/
    cp ${lib.getBin lvm2}/bin/dmsetup $build/bin/
  '' + lib.optionalString withCryptsetup ''
    cp ${cryptsetup}/bin/cryptsetup $build/bin/
  '' + lib.optionalString withKexec ''
    cp ${kexec-tools}/bin/kexec $build/bin/
  '' + lib.optionalString withNvramTool ''
    cp ${nvramtool}/bin/nvramtool $build/bin/
  '' + ''
    runHook postBuild
  '';

  installPhase = ''
    runHook preInstall
    chmod -R u+w $build
    pushd $build
      ${findutils}/bin/find . | ${cpio}/bin/cpio --create -H newc -R +0:+0 > $out
      popd
    runHook postInstall
  '';

  passthru = {
    inherit modules;
  };
}
